function calculatePathArea(pathItem) {
    var area = Math.abs(pathItem.area);
    
    // パスが閉じていない場合は線の面積を追加
    if (!pathItem.closed) {
        var length = pathItem.length;
        var strokeWidth = pathItem.strokeWidth;
        var strokeArea = length * strokeWidth;
        area += strokeArea;
    }

    return area;
}

function calculateCompoundPathArea(compoundPath) {
    var totalArea = 0;

    for (var i = 0; i < compoundPath.pathItems.length; i++) {
        totalArea += calculatePathArea(compoundPath.pathItems[i]);
    }

    return totalArea;
}

function calculateOverlapArea(pathItems) {
    // 全てのパスが同じ点で交差し、同じ線幅を持つと仮定
    var strokeWidth = pathItems[0].strokeWidth;
    var overlapArea = strokeWidth * strokeWidth;

    return overlapArea;
}

function calculateSelectedShapeAreaRatio() {
    if (app.documents.length === 0) {
        alert("ドキュメントが開いていません！");
        return;
    }

    var doc = app.activeDocument;
    var selection = doc.selection;

    if (selection.length !== 1) {
        alert("一つの形状のみを選択してください。");
        return;
    }

    var selectedShape = selection[0];

    // 選択されたオブジェクトが有効なパスアイテムまたはコンパウンドパスアイテムか確認
    var selectedArea;
    if (selectedShape.typename === "CompoundPathItem") {
        selectedArea = calculateCompoundPathArea(selectedShape);
    } else if (selectedShape.typename === "PathItem") {
        selectedArea = calculatePathArea(selectedShape);
    } else {
        alert("選択されたオブジェクトは有効なパスアイテムまたはコンパウンドパスアイテムではありません。");
        return;
    }

    // 選択されたオブジェクトが交差するパスを持つコンパウンドパスアイテムの場合、交差面積を減算
    if (selectedShape.typename === "CompoundPathItem" || selectedShape.typename === "PathItem") {
        var pathItems = (selectedShape.typename === "CompoundPathItem") ? selectedShape.pathItems : [selectedShape];
        var overlapArea = calculateOverlapArea(pathItems);
        selectedArea -= overlapArea;
    }

    // アートボードの面積を計算
    var artboard = doc.artboards[doc.artboards.getActiveArtboardIndex()];
    var artboardRect = artboard.artboardRect;
    var artboardWidth = artboardRect[2] - artboardRect[0];
    var artboardHeight = artboardRect[1] - artboardRect[3];
    var artboardArea = artboardWidth * artboardHeight;

    if (isNaN(artboardArea) || isNaN(selectedArea)) {
        alert("面積の計算に失敗しました。有効なオブジェクトを確認してください。");
        return;
    }

    var ratio = (selectedArea / artboardArea) * 100;

    alert("選択された形状はアートボード全体の " + ratio.toFixed(2) + "% を占めています。\n" +
          "選択された形状の面積: " + selectedArea.toFixed(2) + " px²\n" +
          "アートボードの面積: " + artboardArea.toFixed(2) + " px²");
}

calculateSelectedShapeAreaRatio();
